/*  Pcsx2 - Pc Ps2 Emulator
 *  Copyright (C) 2002-2003  Pcsx2 Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "../Common.h"
#include "Rpc_services.h"

//////////////////version 0x02XX	in XMCSERV
/////////////////////////////////////////////// 0001
void mc2serv_getinfo(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
	struct rMCGETINFO{
		int type;
		int free;
		int pad[34];
		int format;
	} r;
	struct MCGETINFO *in=(struct MCGETINFO*)bin;
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] mc2serv_getinfo(%s, slot=%d, type=%d, free=%d, format=%d, dest=0x%08X)\n", call->send_size,
		in->port?"PORT1":"PORT0", in->slot, in->type, in->free, in->format, in->dest);
#endif
	r.type=2;//PS2 8MB card
    r.free=0;//no free clusters
	r.format=1;//formated
	memcpy(PSM((int)in->dest & 0x1FFFFFFF), &r, sizeof(r));
	*bout=0;
}

/////////////////////////////////////////////// 0002 => 0071
/////////////////////////////////////////////// 0003 => 0072

/////////////////////////////////////////////// 0004
void mc2serv_seek(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] mc2serv_seek()\n", call->send_size);
#endif

}

/////////////////////////////////////////////// 0005
void mc2serv_read(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] mc2serv_read()\n", call->send_size);
#endif

}

/////////////////////////////////////////////// 0006
void mc2serv_write(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] mc2serv_write()\n", call->send_size);
#endif

}

/////////////////////////////////////////////// 00FE => 0070

//////////////////version 0x0101	in MCSERV
/////////////////////////////////////////////// 0070 & 00FE
void mcserv_init(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv] sceMcInit()\n");
#endif
	bout[0]=0;
	if (call->rec_size>=8)	bout[1]=0x20F;	//mcserv.irx version; high, pretending great compatibility;)
	if (call->rec_size>=12)	bout[2]=0x20F;	//mcman.irx version
}

/////////////////////////////////////////////// 0002 & 0071
void mcserv_open(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv] sceMcOpen(port=%d, slot=%d, \"%s\", mode=0x%08X)\n", bin[0], bin[1], bin+5, bin[2]);
#endif
	*bout=4;	//dummy, for test only
}

/////////////////////////////////////////////// 0003 & 0072
void mcserv_close(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv] sceMcClose(fd=%d)\n", bin[0]);
#endif
	*bout=0;
}

/////////////////////////////////////////////// 0073
void mcserv_read(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] sceMcRead()\n", call->send_size);
#endif

}

/////////////////////////////////////////////// 0074
void mcserv_write(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] sceMcWrite()\n", call->send_size);
#endif

}

/////////////////////////////////////////////// 0075
void mcserv_seek(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] sceMcSeek()\n", call->send_size);
#endif

}

/////////////////////////////////////////////// 0076
void mcserv_getdir(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] sceMcGetDir()\n", call->send_size);
#endif

}

/////////////////////////////////////////////// 0077
void mcserv_format(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] sceMcFormat()\n", call->send_size);
#endif

}

/////////////////////////////////////////////// 0078
void mcserv_getinfo(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:mcserv;0x%08X] sceMcGetInfo()\n", call->send_size);
#endif

}

//ENTRYPOINT
void sifcall_mcserv(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
	switch (call->rpc_number) {
//////////////////version 0x02XX
	case MC2_GETINFO:			mc2serv_getinfo(call, bin, bout);	break;
	case MC2_OPEN:				mcserv_open(call, bin, bout);		break;
	case MC2_CLOSE:				mcserv_close(call, bin, bout);		break;
	case MC2_SEEK:				mc2serv_seek(call, bin, bout);		break;
	case MC2_READ:				mc2serv_read(call, bin, bout);		break;
	case MC2_WRITE:				mc2serv_write(call, bin, bout);		break;
/*	case MC2_FLUSH:
	case MC2_MKDIR:
	case MC2_CHDIR:
	case MC2_GETDIR:
	case MC2_SETFILEINFO:
	case MC2_DELETE:
	case MC2_FORMAT:
	case MC2_UNFORMAT:
	case MC2_GETENTSPACE:
	case MC2_RENAME:
	case MC2_CHANGETHREADPRIORITY:*/
	case MC2_INIT:
//////////////////version 0x0101
	case MC_INIT:				mcserv_init(call, bin, bout);		break;
	case MC_OPEN:				mcserv_open(call, bin, bout);		break;
	case MC_CLOSE:				mcserv_close(call, bin, bout);		break;
	case MC_READ:				mcserv_read(call, bin, bout);		break;
	case MC_WRITE:				mcserv_write(call, bin, bout);		break;
	case MC_SEEK:				mcserv_seek(call, bin, bout);		break;
	case MC_GETDIR:				mcserv_getdir(call, bin, bout);		break;
	case MC_FORMAT:				mcserv_format(call, bin, bout);		break;
	case MC_GETINFO:			mcserv_getinfo(call, bin, bout);	break;
	case MC_DELETE:
	case MC_FLUSH:
	case MC_CHDIR:
	case MC_SETFILEINFO:
	case MC_7D:
	case MC_7E:
	case MC_7F:
	case MC_UNFORMAT:
	default:
#ifdef RPC_LOG
		RPC_LOG("[RPC:mcserv] <%04X> function is not implemented\n", call->rpc_number);
#endif
//		*bout=-1;	//general fail
		break;
	}
#ifdef RPC_LOG
//	RPC_LOG("                    0x%08X 0x%08X 0x%08X 0x%08X 0x%08X 0x%08X\n",
//								 bin[0],bin[1],bin[2],bin[3],bin[4],bin[5]);
//	RPC_LOG("                    0x%08X 0x%08X 0x%08X 0x%08X 0x%08X 0x%08X\n",
//								 bin[6],bin[7],bin[8],bin[9],bin[10],bin[11]);
#endif
}