/*  Pcsx2 - Pc Ps2 Emulator
 *  Copyright (C) 2002-2003  Pcsx2 Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>

#include "Rpc_services.h"

/////////////////////////////////////////////// 0000
void cdvdfsv7_searchfile(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
	CdlFILE	*fp;
	char	*name;
	void	*dest;
	int		year, tz;
	struct TocEntry	tocEntry;
		switch(call->send_size){
		case sizeof(struct cdvdfsv_SEARCHFILE):
			fp	=&((struct cdvdfsv_SEARCHFILE*)bin)->fp;
			name=((struct cdvdfsv_SEARCHFILE*)bin)->name;
			dest=((struct cdvdfsv_SEARCHFILE*)bin)->dest;
#ifdef RPC_LOG
			RPC_LOG("[RPC:cdvdfsv7] sceCdSearchFile({lsn=0x%08X, size=0x%08X, \"%s\", data=...}, \"%s\") [old library]\n", fp->lsn, fp->size, fp->name, name);
#endif
			break;
		case sizeof(struct cdvdfsv_SEARCHFILE2):
			fp	=&((struct cdvdfsv_SEARCHFILE2*)bin)->fp;
			name=((struct cdvdfsv_SEARCHFILE2*)bin)->name;
			dest=((struct cdvdfsv_SEARCHFILE2*)bin)->dest;
#ifdef RPC_LOG
			RPC_LOG("[RPC:cdvdfsv7] sceCdSearchFile({lsn=0x%08X, size=0x%08X, \"%s\", data=...}, 0x%08X, \"%s\") [non_dual layer]\n", fp->lsn, fp->size, fp->name, ((struct cdvdfsv_SEARCHFILE2*)bin)->field_20, name);
#endif
			break;
		case sizeof(struct cdvdfsv_SEARCHFILE3):
			fp	=&((struct cdvdfsv_SEARCHFILE3*)bin)->fp;
			name=((struct cdvdfsv_SEARCHFILE3*)bin)->name;
			dest=((struct cdvdfsv_SEARCHFILE3*)bin)->dest;
#ifdef RPC_LOG
			RPC_LOG("[RPC:cdvdfsv7] sceCdSearchFile({lsn=0x%08X, size=0x%08X, \"%s\", data=...}, 0x%08X, \"%s\", layer=%d) [dual layer]\n", fp->lsn, fp->size, fp->name, ((struct cdvdfsv_SEARCHFILE3*)bin)->field_20, name, ((struct cdvdfsv_SEARCHFILE3*)bin)->layer);
#endif
			break;
		default:
#ifdef RPC_LOG
			RPC_LOG("[RPC:cdvdfsv7] sceCdSearchFile(...) [not-supported version; send_size=0x%08X]\n", call->send_size);
#endif
			*bout=0;//not supported
	}

 	*bout=CDVD_findfile(name, &tocEntry);
	fp->lsn=tocEntry.fileLBA;
	fp->size=tocEntry.fileSize;
	strncpy(fp->name, tocEntry.filename, 16);	fp->name[15]=0;
	
	tz		   =tocEntry.date[6];	//timezone	in 1/4 hour units; GMT+2==2*4=8
	if ((tz<-4*13) || (tz>4*13))tz=0;
	tz*=15*60;//convert to seconds
	//tz not used; conversions from minutes, days, years to ...bleah:(
	//this leads to an error of +/-13 hours in date
	
	//dunno if BCD or not; year seems to be BCD
	fp->date[0]=tocEntry.date[5];	//seconds
	fp->date[1]=tocEntry.date[4];	//minutes
	fp->date[2]=tocEntry.date[3];	//hours
	fp->date[3]=tocEntry.date[2];	//day
	fp->date[4]=tocEntry.date[1];	//month
	year	   =tocEntry.date[0]+1900;//year
#define itob(i)         ((i)/10*16 + (i)%10)   /* u_char to BCD */
	fp->date[5]=itob(year/100);
	fp->date[6]=itob(year%100);
	fp->date[7]=0;//not used?!?

	memcmp(PSM((int)dest), fp, sizeof(CdlFILE));
#ifdef RPC_LOG
			RPC_LOG("                     %s FOUND {lsn=0x%08X, size=0x%08X, \"%s\", "
				"data=%02d/%02d/%02X%02X %02d:%02d:%02d tz=GMT+%d}\n", *bout==1 ? "   ":"NOT",
				fp->lsn, fp->size, fp->name,				
				fp->date[3], fp->date[4], fp->date[5], fp->date[6],
				fp->date[2], fp->date[1], fp->date[0], tz/3600);
#endif
}

//ENTRYPOINT
void sifcall_cdvdfsv7(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
	switch (call->rpc_number) {
	case CDVD_SEARCHFILE:	cdvdfsv7_searchfile(call, bin, bout);	break;
	default:
#ifdef RPC_LOG
		RPC_LOG("[RPC:cdvdfsv6] <%04X> function is not implemented\n", call->rpc_number);
#endif
		*bout=0;	//failure
	}
}
